#!/bin/bash

SAVE=0
path="${@: -1}"

CMD=
SUFFIX=

usage()
{
    cat << EOF
    usage: $0 ( -b | -g ) [ -s ] directory

    OPTIONS:
    -b      use bzip2 to compress
    -g      use gzip to compress
    -s      keep zipped files instead of deleting them
EOF
}

if [ $# -lt 2 ];
then
    usage
fi


if [ ! -d "$path" ];
then
    echo "Directory $path does not exist"
    exit -1
fi

while getopts ":hbgs" arg; do
    case $arg in
        h)
            usage
            ;;
        b)
            CMD="tar -jcf"
            ;;
        g)
            CMD="tar -zcf"
            ;;
        s)
            SAVE=1
            ;;
        \?)
            echo "$0 : unrecognized command -$OPTARG" >&2
            ;;
    esac
done

# Action part
cd $path
min=999999999
max=

for fd in *.gray
do
    SUFFIX="$fd.zip $fd"
    $($CMD $SUFFIX)

    res=`du -b $fd.zip`
    array=(`echo $res | tr " " "\n"`)
    size="${array[0]}"

    if [[ "$size" -gt "$max" ]]
    then
        max=$size
    fi

    if [[ "$size" -lt "$min" ]]
    then
        min=$size
    fi

done

span=$(($max - $min))
counter=0

for fd in *.zip
do
    res=`du -b $fd`
    array=(`echo $res | tr " " "\n"`)
    size="${array[0]}"

    diff=$(($size - $min))
    #ratio=$(echo "$diff / $span" | bc -l)
    ratio=$(awk -v diff=$diff -v span=$span 'BEGIN { print diff/span }')

    echo "$counter $ratio"

    counter=$(($counter + 1))

    if [ $SAVE == 0 ];
    then
        rm "$fd"
    fi
done
